@@ -1,6 +1,6 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2021 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2022 Pynguin Contributors
 #
 #  SPDX-License-Identifier: LGPL-3.0-or-later
 #
@@ -9,7 +9,7 @@ from __future__ import annotations
 
 from functools import total_ordering
 from math import inf
-from typing import TYPE_CHECKING, Any, Dict
+from typing import TYPE_CHECKING, Any
 
 import networkx as nx
 
@@ -216,5 +216,5 @@ def _get_node_with_predicate_id(
     return cdg_nodes.pop()
 
 
-def _predicate_fitness(predicate: int, branch_distances: Dict[int, float]) -> float:
+def _predicate_fitness(predicate: int, branch_distances: dict[int, float]) -> float:
     return branch_distances.get(predicate, inf)
