@@ -1,6 +1,6 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2021 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2022 Pynguin Contributors
 #
 #  SPDX-License-Identifier: LGPL-3.0-or-later
 #
@@ -8,7 +8,7 @@
 from __future__ import annotations
 
 import os
-from typing import TYPE_CHECKING, List, Union
+from typing import TYPE_CHECKING
 
 from pynguin.generation.export.abstractexporter import AbstractTestExporter
 
@@ -20,7 +20,5 @@ if TYPE_CHECKING:
 class NoneExporter(AbstractTestExporter):
     """An exporter, which does basically nothing."""
 
-    def export_sequences(
-        self, path: Union[str, os.PathLike], test_cases: List[tc.TestCase]
-    ):
+    def export_sequences(self, path: str | os.PathLike, test_cases: list[tc.TestCase]):
         pass
