@@ -1,6 +1,6 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2021 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2022 Pynguin Contributors
 #
 #  SPDX-License-Identifier: LGPL-3.0-or-later
 #
@@ -13,7 +13,7 @@ import numbers
 import types
 import typing
 from inspect import isclass, isfunction
-from typing import Any, Callable, Optional, Type
+from typing import Any, Callable
 
 from ordered_set import OrderedSet
 from typing_inspect import get_args, get_origin, is_union_type
@@ -25,7 +25,7 @@ PRIMITIVES = OrderedSet([int, str, bytes, bool, float, complex])
 COLLECTIONS = OrderedSet([list, set, tuple, dict])
 
 
-def is_primitive_type(type_: Optional[Type]) -> bool:
+def is_primitive_type(type_: type | None) -> bool:
     """Check if the given type is a primitive.
 
     Args:
@@ -37,7 +37,7 @@ def is_primitive_type(type_: Optional[Type]) -> bool:
     return type_ in PRIMITIVES
 
 
-def is_collection_type(type_: Optional[Type]) -> bool:
+def is_collection_type(type_: type | None) -> bool:
     """Check if the given type is a collection type.
 
     Args:
@@ -75,7 +75,7 @@ def function_in_module(module_name: str) -> Callable[[Any], bool]:
     return lambda member: isfunction(member) and member.__module__ == module_name
 
 
-def is_none_type(type_: Optional[Type]) -> bool:
+def is_none_type(type_: type | None) -> bool:
     """Is the given type NoneType?
 
     Args:
@@ -87,7 +87,7 @@ def is_none_type(type_: Optional[Type]) -> bool:
     return type_ is type(None)  # noqa: E721
 
 
-def is_type_unknown(type_: Optional[Type]) -> bool:
+def is_type_unknown(type_: type | None) -> bool:
     """Is the type of this variable unknown?
 
     Args:
@@ -99,7 +99,7 @@ def is_type_unknown(type_: Optional[Type]) -> bool:
     return type_ is None
 
 
-def is_assignable_to(from_type: Optional[Type], to_type: Optional[Type]) -> bool:
+def is_assignable_to(from_type: type | None, to_type: type | None) -> bool:
     """A naive implementation to check if one type is assignable to another.
 
     Currently only unary types, Any and Union are supported.
@@ -220,9 +220,12 @@ def is_assertable(obj: Any, recursion_depth: int = 0) -> bool:
 
     Primitives (except float) are assertable.
     Enum values are assertable.
-    List, sets, dicts and tuples comprised only of assertable objects are also
+    List, sets, dicts and tuples composed only of assertable objects are also
     assertable.
 
+    Objects that are accepted by this function must be constructable in
+    `pynguin.assertion.assertion_to_ast._create_assertable_object`
+
     Args:
         obj: The object to check for assertability.
         recursion_depth: Avoid endless recursion for nested structures.
@@ -238,7 +241,7 @@ def is_assertable(obj: Any, recursion_depth: int = 0) -> bool:
         return False
 
     tp_ = type(obj)
-    if is_enum(tp_) or is_primitive_type(tp_):
+    if is_enum(tp_) or is_primitive_type(tp_) or is_none_type(tp_):
         return True
     if is_set(obj) or is_list(obj) or is_tuple(obj):
         return all(is_assertable(elem, recursion_depth + 1) for elem in obj)
@@ -251,7 +254,7 @@ def is_assertable(obj: Any, recursion_depth: int = 0) -> bool:
     return False
 
 
-def get_class_that_defined_method(method: object) -> Optional[object]:
+def get_class_that_defined_method(method: object) -> object | None:
     """Retrieves the class that defines a method.
 
     Taken from https://stackoverflow.com/a/25959545/4293396
@@ -304,7 +307,7 @@ def is_optional_parameter(inf_sig: InferredSignature, parameter_name: str) -> bo
     )
 
 
-def wrap_var_param_type(type_: Optional[type], param_kind) -> Optional[type]:
+def wrap_var_param_type(type_: type | None, param_kind) -> type | None:
     """Wrap the parameter type of *args and **kwargs in List[...] or Dict[str, ...],
     respectively.
 
